from __future__ import annotations

from datetime import timedelta
from functools import cached_property
from importlib import import_module
from typing import TYPE_CHECKING

from django.conf import settings
from django.db.models import F
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import (
    OpenApiParameter,
    extend_schema,
    extend_schema_view,
)
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.filters import SearchFilter
from rest_framework.response import Response

from blog.filters.post import BlogPostFilter
from blog.models.post import BlogPost
from blog.serializers.comment import BlogCommentSerializer
from blog.serializers.post import (
    BlogPostDetailSerializer,
    BlogPostLikedPostsRequestSerializer,
    BlogPostLikedPostsResponseSerializer,
    BlogPostSerializer,
    BlogPostWriteSerializer,
)
from blog.strategies.weighted_related_posts_strategy import (
    WeightedRelatedPostsStrategy,
)
from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from core.utils.views import cache_methods

if TYPE_CHECKING:
    from blog.strategies.related_posts_strategy import RelatedPostsStrategy


@extend_schema_view(
    **create_schema_view_config(
        model_class=BlogPost,
        display_config={
            "tag": "Blog Post",
        },
        serializers={
            "list_serializer": BlogPostSerializer,
            "detail_serializer": BlogPostDetailSerializer,
            "write_serializer": BlogPostWriteSerializer,
        },
        error_serializer=ErrorResponseSerializer,
        additional_responses={
            "create": {201: BlogPostDetailSerializer},
            "update": {200: BlogPostDetailSerializer},
            "partial_update": {200: BlogPostDetailSerializer},
        },
    )
)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class BlogPostViewSet(MultiSerializerMixin, BaseModelViewSet):
    queryset = BlogPost.objects.all()
    serializers = {
        "default": BlogPostDetailSerializer,
        "list": BlogPostSerializer,
        "retrieve": BlogPostDetailSerializer,
        "create": BlogPostWriteSerializer,
        "update": BlogPostWriteSerializer,
        "partial_update": BlogPostWriteSerializer,
        "update_likes": BlogPostLikedPostsResponseSerializer,
        "update_view_count": BlogPostDetailSerializer,
        "liked_posts": BlogPostLikedPostsResponseSerializer,
        "related_posts": BlogPostSerializer,
        "comments": BlogCommentSerializer,
        "trending": BlogPostSerializer,
        "popular": BlogPostSerializer,
        "featured": BlogPostSerializer,
    }
    response_serializers = {
        "create": BlogPostDetailSerializer,
        "update": BlogPostDetailSerializer,
        "partial_update": BlogPostDetailSerializer,
    }
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    filterset_class = BlogPostFilter
    ordering_fields = [
        "id",
        "created_at",
        "updated_at",
        "published_at",
        "view_count",
        "likes_count",
        "comments_count",
        "tags_count",
        "featured",
    ]
    ordering = ["-created_at"]
    search_fields = [
        "translations__title",
        "translations__subtitle",
        "translations__body",
        "category__translations__name",
        "tags__translations__name",
    ]

    def get_queryset(self):
        queryset = (
            BlogPost.objects.select_related(
                "author__user", "category", "author"
            )
            .prefetch_related(
                "tags__translations",
                "likes",
                "comments",
                "translations",
                "category__translations",
            )
            .with_all_annotations()
        )
        return queryset

    @cached_property
    def related_posts_strategy(self):
        strategies_with_weights: list[tuple[RelatedPostsStrategy, float]] = []
        for strategy_config in settings.RELATED_POSTS_STRATEGIES:
            strategy_path = strategy_config["strategy"]
            weight = strategy_config["weight"]
            module_path, class_name = strategy_path.rsplit(".", 1)
            module = import_module(module_path)
            strategy_class = getattr(module, class_name)
            strategy_instance = strategy_class()
            strategies_with_weights.append((strategy_instance, weight))

        limit = getattr(settings, "RELATED_POSTS_LIMIT", 8)
        return WeightedRelatedPostsStrategy(
            strategies_with_weights, limit=limit
        )

    def get_related_posts_strategy(self):
        return self.related_posts_strategy

    @extend_schema(
        operation_id="toggleBlogPostLike",
        summary=_("Toggle post like"),
        description=_("Like or unlike a blog post. Toggles the like status."),
        tags=["Blog Posts"],
        request=BlogPostLikedPostsRequestSerializer,
        responses={
            200: BlogPostLikedPostsResponseSerializer,
            401: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["POST"])
    def update_likes(self, request, pk=None):
        if not request.user.is_authenticated:
            return Response(
                {"detail": _("Authentication credentials were not provided.")},
                status=status.HTTP_401_UNAUTHORIZED,
            )

        post = self.get_object()
        user = request.user

        if post.likes.filter(pk=user.pk).exists():
            post.likes.remove(user)
        else:
            post.likes.add(user)
        post.save()

        serializer = BlogPostDetailSerializer(
            post, context=self.get_serializer_context()
        )
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        operation_id="incrementBlogPostViews",
        summary=_("Increment post view count"),
        description=_("Increment the view count for a blog post."),
        tags=["Blog Posts"],
        responses={
            200: BlogPostDetailSerializer,
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["POST"])
    def update_view_count(self, request, pk=None):
        post = self.get_object()
        post.view_count += 1
        post.save(update_fields=["view_count"])
        serializer = self.get_serializer(post)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        operation_id="listBlogPostRelated",
        summary=_("Get related posts"),
        description=_("Get related posts for a blog post."),
        tags=["Blog Posts"],
        responses={
            200: BlogPostSerializer(many=True),
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["GET"])
    def related_posts(self, request, pk=None):
        post = self.get_object()
        strategy = self.get_related_posts_strategy()
        related_posts = strategy.get_related_posts(post)

        serializer = self.get_serializer(related_posts, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        operation_id="checkBlogPostLikes",
        summary=_("Get liked posts"),
        description=_("Get all posts that the authenticated user has liked."),
        tags=["Blog Posts"],
        request=BlogPostLikedPostsRequestSerializer,
        responses={
            200: BlogPostLikedPostsResponseSerializer,
            400: ErrorResponseSerializer,
        },
    )
    @action(detail=False, methods=["POST"])
    def liked_posts(self, request, *args, **kwargs):
        serializer = BlogPostLikedPostsRequestSerializer(data=request.data)
        if not serializer.is_valid():
            return Response(
                serializer.errors, status=status.HTTP_400_BAD_REQUEST
            )

        user = request.user
        post_ids = serializer.validated_data["post_ids"]

        liked_post_ids = BlogPost.objects.filter(
            likes=user, id__in=post_ids
        ).values_list("id", flat=True)

        response_data = {"post_ids": list(liked_post_ids)}
        response_serializer = BlogPostLikedPostsResponseSerializer(
            response_data
        )
        return Response(response_serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        operation_id="listBlogPostComments",
        summary=_("Get post comments"),
        description=_("Get all comments for a blog post."),
        tags=["Blog Posts"],
        responses={
            200: BlogCommentSerializer(many=True),
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["GET"])
    def comments(self, request, pk=None):
        post = self.get_object()
        queryset = post.comments.select_related(
            "user", "parent"
        ).prefetch_related("translations", "likes")

        parent_id = request.query_params.get("parent", None)
        if parent_id is not None:
            if parent_id.lower() == "none":
                queryset = queryset.filter(parent__isnull=True)
            else:
                try:
                    queryset = queryset.filter(parent_id=int(parent_id))
                except (ValueError, TypeError):
                    return Response(
                        {"detail": _("Invalid parent ID.")},
                        status=status.HTTP_400_BAD_REQUEST,
                    )

        return self.paginate_and_serialize(queryset, request)

    @extend_schema(
        operation_id="listTrendingBlogPosts",
        summary=_("Get trending posts"),
        description=_(
            "Get trending blog posts based on recent engagement metrics. "
            "Combines views, likes, and comments from recent time period."
        ),
        tags=["Blog Posts"],
        parameters=[
            OpenApiParameter(
                name="days",
                type=int,
                location=OpenApiParameter.QUERY,
                description="Number of days to look back for trending calculation",
                required=False,
                default=7,
            ),
        ],
        responses={
            200: BlogPostSerializer(many=True),
        },
    )
    @action(detail=False, methods=["GET"])
    def trending(self, request):
        days = int(request.query_params.get("days", 7))
        cutoff_date = timezone.now() - timedelta(days=days)

        queryset = (
            self.get_queryset()
            .filter(published_at__gte=cutoff_date)
            .annotate(
                trending_score=F("view_count")
                + (F("likes_count_field") * 2)
                + (F("comments_count_field") * 3)
            )
            .order_by("-trending_score")
        )

        return self.paginate_and_serialize(queryset, request)

    @extend_schema(
        operation_id="listPopularBlogPosts",
        summary=_("Get popular posts"),
        description=_(
            "Get most popular blog posts based on all-time engagement metrics."
        ),
        tags=["Blog Posts"],
        responses={
            200: BlogPostSerializer(many=True),
        },
    )
    @action(detail=False, methods=["GET"])
    def popular(self, request):
        queryset = self.get_queryset().order_by(
            "-likes_count_field", "-view_count"
        )
        return self.paginate_and_serialize(queryset, request)

    @extend_schema(
        operation_id="listFeaturedBlogPosts",
        summary=_("Get featured posts"),
        description=_(
            "Get posts marked as featured, ordered by publication date."
        ),
        tags=["Blog Posts"],
        responses={
            200: BlogPostSerializer(many=True),
        },
    )
    @action(detail=False, methods=["GET"])
    def featured(self, request):
        queryset = (
            self.get_queryset().filter(featured=True).order_by("-published_at")
        )
        return self.paginate_and_serialize(queryset, request)



from drf_spectacular.utils import extend_schema_field
from parler_rest.serializers import (
    TranslatableModelSerializer,
)

from core.api.schema import generate_schema_multi_lang
from core.utils.serializers import TranslatedFieldExtended
from tag.models import Tag


@extend_schema_field(generate_schema_multi_lang(Tag))
class TranslatedFieldsFieldExtend(TranslatedFieldExtended):
    pass


class TagSerializer(TranslatableModelSerializer):
    translations = TranslatedFieldsFieldExtend(shared_model=Tag)

    class Meta:
        model = Tag
        fields = [
            "id",
            "translations",
            "active",
            "sort_order",
            "created_at",
            "updated_at",
            "uuid",
        ]
        read_only_fields = (
            "created_at",
            "updated_at",
            "uuid",
        )



from datetime import timedelta

from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from cart.models import CartItem


class CartItemFilter(filters.FilterSet):
    id = filters.NumberFilter(
        field_name="id",
        lookup_expr="exact",
        help_text=_("Filter by cart item ID"),
    )
    cart = filters.NumberFilter(
        field_name="cart__id",
        lookup_expr="exact",
        help_text=_("Filter by cart ID"),
    )
    product = filters.NumberFilter(
        field_name="product__id",
        lookup_expr="exact",
        help_text=_("Filter by product ID"),
    )
    product_name = filters.CharFilter(
        field_name="product__translations__name",
        lookup_expr="icontains",
        help_text=_("Filter by product name (partial match)"),
    )
    user = filters.NumberFilter(
        field_name="cart__user__id",
        lookup_expr="exact",
        help_text=_("Filter by user ID"),
    )
    user_email = filters.CharFilter(
        field_name="cart__user__email",
        lookup_expr="icontains",
        help_text=_("Filter by user email (partial match)"),
    )

    min_quantity = filters.NumberFilter(
        field_name="quantity",
        lookup_expr="gte",
        help_text=_("Filter by minimum quantity"),
    )
    max_quantity = filters.NumberFilter(
        field_name="quantity",
        lookup_expr="lte",
        help_text=_("Filter by maximum quantity"),
    )

    min_price = filters.NumberFilter(
        field_name="product__price__amount",
        lookup_expr="gte",
        help_text=_("Filter by minimum product price"),
    )
    max_price = filters.NumberFilter(
        field_name="product__price__amount",
        lookup_expr="lte",
        help_text=_("Filter by maximum product price"),
    )

    in_active_carts = filters.BooleanFilter(
        method="filter_active_carts",
        help_text=_("Filter items in active carts"),
    )

    with_discounts = filters.BooleanFilter(
        method="filter_with_discounts",
        help_text=_("Filter items with product discounts"),
    )

    def filter_active_carts(self, queryset, name, value):
        if value:
            cutoff_time = timezone.now() - timedelta(hours=24)
            return queryset.filter(cart__last_activity__gte=cutoff_time)
        else:
            cutoff_time = timezone.now() - timedelta(hours=24)
            return queryset.filter(cart__last_activity__lt=cutoff_time)

    def filter_with_discounts(self, queryset, name, value):
        if value:
            return queryset.filter(product__discount_percent__gt=0)
        else:
            return queryset.filter(product__discount_percent=0)

    class Meta:
        model = CartItem
        fields = [
            "id",
            "cart",
            "product",
            "product_name",
            "user",
            "user_email",
            "min_quantity",
            "max_quantity",
            "min_price",
            "max_price",
            "in_active_carts",
            "with_discounts",
        ]



from asgiref.sync import sync_to_async
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db.models.signals import m2m_changed
from django.dispatch import receiver

from blog.models import BlogComment
from notification.enum import NotificationKindEnum
from notification.models.notification import Notification
from notification.models.user import NotificationUser

User = get_user_model()

languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]


@receiver(m2m_changed, sender=BlogComment.likes.through)
async def notify_comment_liked(
    sender, instance, action, reverse, pk_set, **kwargs
):
    if action == "post_add" and not reverse:
        post = await sync_to_async(lambda: instance.post)()
        user = await sync_to_async(lambda: instance.user)()
        blog_post_url = f"{settings.NUXT_BASE_URL}/blog/post/{post.id}/{post.slug}#blog-post-comments"

        for user_id in pk_set:
            if user and user.id != user_id:
                liker_user = await User.objects.aget(id=user_id)
                notification = await Notification.objects.acreate(
                    kind=NotificationKindEnum.INFO,
                    link=blog_post_url,
                )

                for language in languages:
                    await sync_to_async(notification.set_current_language)(
                        language
                    )
                    if language == "en":
                        await sync_to_async(setattr)(
                            notification,
                            "title",
                            f"<a href='{blog_post_url}'>Comment</a> Liked!",
                        )
                        await sync_to_async(setattr)(
                            notification,
                            "message",
                            f"Your comment was liked by "
                            f"{liker_user.username if liker_user.username else liker_user.email}.",
                        )
                    elif language == "el":
                        await sync_to_async(setattr)(
                            notification,
                            "title",
                            f"Το <a href="  # noqa: RUF001
                            f"'"
                            f"{blog_post_url}'>σχόλιο</a> "
                            f"σου πήρε like!",  # noqa: RUF001
                        )
                        await sync_to_async(setattr)(
                            notification,
                            "message",
                            f"Το σχόλιο σου άρεσε στον χρήστη "  # noqa: RUF001
                            f"{liker_user.username if liker_user.username else liker_user.email}.",
                        )
                    await notification.asave()

                await NotificationUser.objects.acreate(
                    user=user, notification=notification
                )



from datetime import timedelta

from django.contrib import admin
from django.utils import timezone
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from unfold.admin import ModelAdmin, TabularInline
from unfold.contrib.filters.admin import (
    DropdownFilter,
    RangeDateTimeFilter,
    RangeNumericListFilter,
    RelatedDropdownFilter,
    SliderNumericFilter,
)
from unfold.decorators import action
from unfold.enums import ActionVariant

from cart.models import Cart, CartItem


class CartTypeFilter(DropdownFilter):
    title = _("Cart Type")
    parameter_name = "cart_type"

    def lookups(self, request, model_admin):
        return [
            ("authenticated", _("Authenticated Users")),
            ("guest", _("Guest Users")),
            ("abandoned", _("Abandoned Carts")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "authenticated":
            return queryset.filter(user__isnull=False)
        elif self.value() == "guest":
            return queryset.filter(user__isnull=True, session_key__isnull=False)
        elif self.value() == "abandoned":
            cutoff = timezone.now() - timedelta(hours=24)
            return queryset.filter(last_activity__lt=cutoff)
        return queryset


class TotalItemsFilter(RangeNumericListFilter):
    title = _("Total Items")
    parameter_name = "total_items"

    def queryset(self, request, queryset):
        filters = {}

        value_from = self.used_parameters.get(f"{self.parameter_name}_from")
        if value_from and value_from != "":
            filters["items__quantity__sum__gte"] = value_from

        value_to = self.used_parameters.get(f"{self.parameter_name}_to")
        if value_to and value_to != "":
            filters["items__quantity__sum__lte"] = value_to

        return queryset.filter(**filters) if filters else queryset

    def expected_parameters(self):
        return [
            f"{self.parameter_name}_from",
            f"{self.parameter_name}_to",
        ]


class ActivityStatusFilter(DropdownFilter):
    title = _("Activity Status")
    parameter_name = "activity_status"

    def lookups(self, request, model_admin):
        return [
            ("active", _("Active (Last 24h)")),
            ("recent", _("Recent (Last 7 days)")),
            ("old", _("Old (7+ days)")),
        ]

    def queryset(self, request, queryset):
        now = timezone.now()
        if self.value() == "active":
            return queryset.filter(last_activity__gte=now - timedelta(hours=24))
        elif self.value() == "recent":
            return queryset.filter(
                last_activity__gte=now - timedelta(days=7),
                last_activity__lt=now - timedelta(hours=24),
            )
        elif self.value() == "old":
            return queryset.filter(last_activity__lt=now - timedelta(days=7))
        return queryset


class CartItemInline(TabularInline):
    model = CartItem
    extra = 0
    fields = (
        "product_display",
        "quantity",
        "unit_price_display",
        "total_price_display",
        "discount_info",
    )
    readonly_fields = (
        "product_display",
        "unit_price_display",
        "total_price_display",
        "discount_info",
    )

    tab = True
    show_change_link = True

    def product_display(self, obj):
        if obj.product:
            product_name = (
                obj.product.safe_translation_getter("name", any_language=True)
                or "Unnamed Product"
            )
            return format_html(
                '<div class="text-sm">'
                '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
                '<div class="text-base-500 dark:text-base-400">ID: {}</div>'
                "</div>",
                product_name,
                obj.product.id,
            )
        return "-"

    product_display.short_description = _("Product")

    def unit_price_display(self, obj):
        if hasattr(obj, "price") and hasattr(obj, "final_price"):
            if obj.price != obj.final_price:
                return format_html(
                    '<div class="text-sm">'
                    '<div class="line-through text-base-500 dark:text-base-400">{}</div>'
                    '<div class="font-medium text-green-600 dark:text-green-400">{}</div>'
                    "</div>",
                    obj.price,
                    obj.final_price,
                )
            else:
                return format_html(
                    '<div class="text-sm font-medium text-base-900 dark:text-base-100">{}</div>',
                    obj.final_price,
                )
        return "-"

    unit_price_display.short_description = _("Unit Price")

    def total_price_display(self, obj):
        if hasattr(obj, "total_price"):
            return format_html(
                '<div class="text-sm font-bold text-base-900 dark:text-base-100">{}</div>',
                obj.total_price,
            )
        return "-"

    total_price_display.short_description = _("Total")

    def discount_info(self, obj):
        if hasattr(obj, "discount_percent") and obj.discount_percent > 0:
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full">'
                "-{}%"
                "</span>",
                obj.discount_percent,
            )
        return ""

    discount_info.short_description = _("Discount")


@admin.register(Cart)
class CartAdmin(ModelAdmin):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = True
    list_filter_submit = True
    list_filter_sheet = True

    list_display = (
        "id",
        "cart_owner_display",
        "cart_type_badge",
        "activity_status_badge",
        "items_summary",
        "price_summary",
        "last_activity",
        "created_at",
    )
    list_filter = (
        CartTypeFilter,
        ActivityStatusFilter,
        ("last_activity", RangeDateTimeFilter),
        ("created_at", RangeDateTimeFilter),
        ("user", RelatedDropdownFilter),
        TotalItemsFilter,
    )
    search_fields = (
        "id",
        "session_key",
        "user__email",
        "user__username",
        "user__first_name",
        "user__last_name",
    )
    date_hierarchy = "last_activity"
    list_select_related = ["user"]
    readonly_fields = [
        "id",
        "session_key",
        "created_at",
        "updated_at",
        "last_activity",
        "cart_summary",
        "financial_summary",
    ]

    fieldsets = (
        (
            _("Cart Owner"),
            {
                "fields": ("user", "session_key"),
                "classes": ("wide",),
            },
        ),
        (
            _("Activity"),
            {
                "fields": ("last_activity", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
        (
            _("Summary"),
            {
                "fields": ("cart_summary", "financial_summary"),
                "classes": ("collapse",),
            },
        ),
    )

    inlines = [CartItemInline]

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.prefetch_related("items__product")

    def cart_owner_display(self, obj):
        if obj.user:
            full_name = obj.user.full_name
            username = obj.user.username
            email = obj.user.email

            display_name = full_name if full_name else username
            return format_html(
                '<div class="text-sm">'
                '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
                '<div class="text-base-500 dark:text-base-400">{}</div>'
                "</div>",
                display_name,
                email,
            )
        elif obj.session_key:
            return format_html(
                '<div class="text-sm">'
                '<div class="font-medium text-base-700 dark:text-base-300">Guest User</div>'
                '<div class="text-base-500 dark:text-base-400 font-mono">{}...</div>'
                "</div>",
                obj.session_key[:12],
            )
        return format_html(
            '<span class="text-base-400 dark:text-base-500 italic">Anonymous</span>'
        )

    cart_owner_display.short_description = _("Cart Owner")

    def cart_type_badge(self, obj):
        if obj.user:
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 rounded-full">'
                "👤 User"
                "</span>"
            )
        elif obj.session_key:
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-300 rounded-full">'
                "🛒 Guest"
                "</span>"
            )
        return format_html(
            '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-gray-50 dark:bg-gray-800 text-base-700 dark:text-base-300 rounded-full">'
            "❓ Unknown"
            "</span>"
        )

    cart_type_badge.short_description = _("Type")

    def activity_status_badge(self, obj):
        now = timezone.now()
        time_diff = now - obj.last_activity

        if time_diff < timedelta(hours=1):
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 rounded-full">'
                "🟢 Active"
                "</span>"
            )
        elif time_diff < timedelta(hours=24):
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-300 rounded-full">'
                "🟡 Recent"
                "</span>"
            )
        elif time_diff < timedelta(days=7):
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-orange-50 dark:bg-orange-900 text-orange-700 dark:text-orange-300 rounded-full">'
                "🟠 Idle"
                "</span>"
            )
        else:
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full">'
                "🔴 Abandoned"
                "</span>"
            )

    activity_status_badge.short_description = _("Activity")

    def items_summary(self, obj):
        total_items = obj.total_items
        unique_items = obj.total_items_unique

        return format_html(
            '<div class="text-sm text-base-700 dark:text-base-300">'
            '<div class="font-medium">{} items</div>'
            '<div class="text-base-500 dark:text-base-400">{} unique</div>'
            "</div>",
            total_items,
            unique_items,
        )

    items_summary.short_description = _("Items")

    def price_summary(self, obj):
        total_price = obj.total_price
        total_discount = obj.total_discount_value

        if total_discount.amount > 0:
            return format_html(
                '<div class="text-sm">'
                '<div class="font-bold text-base-900 dark:text-base-100">{}</div>'
                '<div class="text-red-600 dark:text-red-400 text-xs">-{} saved</div>'
                "</div>",
                total_price,
                total_discount,
            )
        else:
            return format_html(
                '<div class="text-sm font-bold text-base-900 dark:text-base-100">{}</div>',
                total_price,
            )

    price_summary.short_description = _("Total")

    def cart_summary(self, obj):
        return format_html(
            '<div class="grid grid-cols-2 gap-4 text-sm">'
            "<div>"
            "<strong>Items:</strong> {} total, {} unique<br>"
            "<strong>Total Price:</strong> {}<br>"
            "<strong>Total Discount:</strong> {}"
            "</div>"
            "<div>"
            "<strong>VAT:</strong> {}<br>"
            "<strong>Activity:</strong> {}<br>"
            "<strong>Created:</strong> {}"
            "</div>"
            "</div>",
            obj.total_items,
            obj.total_items_unique,
            obj.total_price,
            obj.total_discount_value,
            obj.total_vat_value,
            obj.last_activity.strftime("%Y-%m-%d %H:%M"),
            obj.created_at.strftime("%Y-%m-%d %H:%M"),
        )

    cart_summary.short_description = _("Cart Summary")

    def financial_summary(self, obj):
        savings_percent = 0
        if obj.total_price.amount > 0 and obj.total_discount_value.amount > 0:
            original_total = (
                obj.total_price.amount + obj.total_discount_value.amount
            )
            savings_percent = (
                obj.total_discount_value.amount / original_total
            ) * 100

        return format_html(
            '<div class="text-sm">'
            '<div class="mb-2"><strong>Financial Breakdown:</strong></div>'
            '<div class="grid grid-cols-2 gap-2">'
            '<div>Final Total:</div><div class="font-bold">{}</div>'
            '<div>Total Discounts:</div><div class="text-red-600 dark:text-red-400">-{}</div>'
            "<div>Total VAT:</div><div>{}</div>"
            '<div>Savings:</div><div class="text-green-600 dark:text-green-400">{:.1f}%</div>'
            "</div>"
            "</div>",
            obj.total_price,
            obj.total_discount_value,
            obj.total_vat_value,
            savings_percent,
        )

    financial_summary.short_description = _("Financial Summary")


@admin.register(CartItem)
class CartItemAdmin(ModelAdmin):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = True
    list_filter_submit = True
    list_filter_sheet = True

    list_display = (
        "id",
        "cart_info",
        "product_display",
        "quantity_display",
        "pricing_info",
        "discount_badge",
        "created_at",
    )
    list_filter = (
        ("cart", RelatedDropdownFilter),
        ("product", RelatedDropdownFilter),
        ("quantity", SliderNumericFilter),
        ("created_at", RangeDateTimeFilter),
    )
    search_fields = (
        "cart__id",
        "cart__user__email",
        "cart__user__username",
        "product__translations__name",
        "cart__session_key",
    )
    list_select_related = ["cart", "cart__user", "product"]
    readonly_fields = [
        "id",
        "created_at",
        "updated_at",
        "pricing_breakdown",
        "savings_info",
    ]

    fieldsets = (
        (
            _("Cart Item"),
            {
                "fields": ("cart", "product", "quantity"),
                "classes": ("wide",),
            },
        ),
        (
            _("Pricing Information"),
            {
                "fields": ("pricing_breakdown", "savings_info"),
                "classes": ("collapse",),
            },
        ),
        (
            _("System"),
            {
                "fields": ("id", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    actions = [
        "increase_quantity",
        "decrease_quantity",
        "remove_from_cart",
    ]

    def cart_info(self, obj):
        owner_info = (
            "Guest"
            if not obj.cart.user
            else obj.cart.user.full_name or obj.cart.user.username
        )
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">Cart #{}</div>'
            '<div class="text-base-500 dark:text-base-400">{}</div>'
            "</div>",
            obj.cart.id,
            owner_info,
        )

    cart_info.short_description = _("Cart")

    def product_display(self, obj):
        product_name = (
            obj.product.safe_translation_getter("name", any_language=True)
            or "Unnamed Product"
        )
        product_id = obj.product.id

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="text-base-500 dark:text-base-400">ID: {}</div>'
            "</div>",
            product_name,
            product_id,
        )

    product_display.short_description = _("Product")

    def quantity_display(self, obj):
        return format_html(
            '<span class="inline-flex items-center px-3 py-1 text-sm font-medium bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-300 rounded-full">'
            "x{}"
            "</span>",
            obj.quantity,
        )

    quantity_display.short_description = _("Qty")

    def pricing_info(self, obj):
        if (
            hasattr(obj, "price")
            and hasattr(obj, "final_price")
            and hasattr(obj, "total_price")
        ):
            if obj.price != obj.final_price:
                return format_html(
                    '<div class="text-sm">'
                    '<div class="text-base-500 dark:text-base-400 line-through">{} each</div>'
                    '<div class="font-medium text-green-600 dark:text-green-400">{} each</div>'
                    '<div class="font-bold text-base-900 dark:text-base-100">Total: {}</div>'
                    "</div>",
                    obj.price,
                    obj.final_price,
                    obj.total_price,
                )
            else:
                return format_html(
                    '<div class="text-sm">'
                    '<div class="font-medium text-base-900 dark:text-base-100">{} each</div>'
                    '<div class="font-bold text-base-900 dark:text-base-100">Total: {}</div>'
                    "</div>",
                    obj.final_price,
                    obj.total_price,
                )
        return "-"

    pricing_info.short_description = _("Pricing")

    def discount_badge(self, obj):
        if hasattr(obj, "discount_percent") and obj.discount_percent > 0:
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full">'
                "🏷️ -{}%"
                "</span>",
                obj.discount_percent,
            )
        return ""

    discount_badge.short_description = _("Discount")

    def pricing_breakdown(self, obj):
        return format_html(
            '<div class="text-sm">'
            '<div class="grid grid-cols-2 gap-2">'
            "<div>Unit Price:</div><div>{}</div>"
            '<div>Final Price:</div><div class="font-medium">{}</div>'
            "<div>Quantity:</div><div>{}</div>"
            '<div>Total:</div><div class="font-bold">{}</div>'
            "</div>"
            "</div>",
            getattr(obj, "price", "N/A"),
            getattr(obj, "final_price", "N/A"),
            obj.quantity,
            getattr(obj, "total_price", "N/A"),
        )

    pricing_breakdown.short_description = _("Pricing Breakdown")

    def savings_info(self, obj):
        if hasattr(obj, "discount_percent") and obj.discount_percent > 0:
            return format_html(
                '<div class="text-sm text-green-600 dark:text-green-400">'
                "<div>Discount: {}%</div>"
                "<div>You save: {} per item</div>"
                "<div>Total savings: {}</div>"
                "</div>",
                getattr(obj, "discount_percent", 0),
                getattr(obj, "discount_value", "N/A"),
                getattr(obj, "total_discount_value", "N/A"),
            )
        return format_html(
            '<div class="text-sm text-base-500 dark:text-base-400">No discounts applied</div>'
        )

    savings_info.short_description = _("Savings")

    @action(
        description=_("Increase quantity by 1"),
        variant=ActionVariant.PRIMARY,
        icon="add",
    )
    def increase_quantity(self, request, queryset):
        for item in queryset:
            item.quantity += 1
            item.save()
        self.message_user(
            request,
            _("Quantity increased for %(count)d items.")
            % {"count": queryset.count()},
        )

    @action(
        description=_("Decrease quantity by 1"),
        variant=ActionVariant.WARNING,
        icon="remove",
    )
    def decrease_quantity(self, request, queryset):
        for item in queryset:
            if item.quantity > 1:
                item.quantity -= 1
                item.save()
            else:
                item.delete()
        self.message_user(
            request,
            _("Quantity decreased for %(count)d items.")
            % {"count": queryset.count()},
        )

    @action(
        description=_("Remove from cart"),
        variant=ActionVariant.DANGER,
        icon="delete",
    )
    def remove_from_cart(self, request, queryset):
        count = queryset.count()
        queryset.delete()
        self.message_user(
            request,
            _("%(count)d items were removed from carts.") % {"count": count},
        )



import importlib
import random
import string

import factory
from django.apps import apps
from django.conf import settings
from faker import Faker

from core.factories import CustomDjangoModelFactory
from country.models import Country
from region.factories import RegionFactory

fake = Faker()

available_languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]


def generate_unique_country_codes():
    try:
        country_model = importlib.import_module("country.models").Country
        existing_alpha_2_codes = set(
            country_model.objects.values_list("alpha_2", flat=True)
        )
        existing_alpha_3_codes = set(
            country_model.objects.values_list("alpha_3", flat=True)
        )

        for _ in range(100):
            try:
                alpha_2_code = fake.country_code(representation="alpha-2")
                alpha_3_code = fake.country_code(representation="alpha-3")
                if (
                    alpha_2_code not in existing_alpha_2_codes
                    and alpha_3_code not in existing_alpha_3_codes
                ):
                    return alpha_2_code, alpha_3_code
            except Exception:
                alpha_2_code = "".join(
                    random.choices(string.ascii_uppercase, k=2)
                )
                alpha_3_code = "".join(
                    random.choices(string.ascii_uppercase, k=3)
                )
                if (
                    alpha_2_code not in existing_alpha_2_codes
                    and alpha_3_code not in existing_alpha_3_codes
                ):
                    return alpha_2_code, alpha_3_code

        alpha_2_code = "".join(random.choices(string.ascii_uppercase, k=2))
        alpha_3_code = "".join(random.choices(string.ascii_uppercase, k=3))
        return alpha_2_code, alpha_3_code

    except Exception:
        alpha_2_code = "".join(random.choices(string.ascii_uppercase, k=2))
        alpha_3_code = "".join(random.choices(string.ascii_uppercase, k=3))
        return alpha_2_code, alpha_3_code


class CountryTranslationFactory(factory.django.DjangoModelFactory):
    language_code = factory.Iterator(available_languages)
    name = factory.Faker("country")
    master = factory.SubFactory("country.factories.CountryFactory")

    class Meta:
        model = apps.get_model("country", "CountryTranslation")
        django_get_or_create = ("language_code", "master")


class CountryFactory(CustomDjangoModelFactory):
    unique_model_fields = [
        ("alpha_2", lambda: generate_unique_country_codes()[0]),
        ("alpha_3", lambda: generate_unique_country_codes()[1]),
        ("iso_cc", lambda: fake.random_int(min=1, max=999)),
        ("phone_code", lambda: fake.random_int(min=1, max=999)),
    ]

    image_flag = factory.django.ImageField(
        filename=factory.LazyAttribute(lambda _: f"{fake.word()}.png"),
        color=factory.Faker("color"),
        width=640,
        height=480,
    )

    class Meta:
        model = Country
        django_get_or_create = ("alpha_2",)
        skip_postgeneration_save = True

    @factory.post_generation
    def num_regions(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            RegionFactory.create_batch(extracted, country=self)

    @factory.post_generation
    def translations(self, create, extracted, **kwargs):
        if not create:
            return

        translations = extracted or [
            CountryTranslationFactory(language_code=lang, master=self)
            for lang in available_languages
        ]

        for translation in translations:
            translation.save()



from djangorestframework_camel_case.parser import (
    CamelCaseFormParser,
    CamelCaseJSONParser,
    CamelCaseMultiPartParser,
)


class NoUnderscoreBeforeNumberCamelCaseJSONParser(CamelCaseJSONParser):
    json_underscoreize = {"no_underscore_before_number": True}


class NoUnderscoreBeforeNumberCamelCaseFormParser(CamelCaseFormParser):
    def parse(self, stream, media_type=None, parser_context=None):
        return super().parse(
            stream,
            media_type,
            {
                **parser_context,
                "json_underscoreize": {
                    "no_underscore_before_number": True,
                    "ignore_keys": (
                        "password1",
                        "password2",
                        "new_password1",
                        "new_password2",
                    ),
                    "ignore_fields": (
                        "password1",
                        "password2",
                        "new_password1",
                        "new_password2",
                    ),
                },
            },
        )


class NoUnderscoreBeforeNumberCamelCaseMultiPartParser(
    CamelCaseMultiPartParser
):
    def parse(self, stream, media_type=None, parser_context=None):
        return super().parse(
            stream,
            media_type,
            {
                **parser_context,
                "json_underscoreize": {
                    "no_underscore_before_number": True,
                    "ignore_keys": (
                        "password1",
                        "password2",
                        "new_password1",
                        "new_password2",
                    ),
                    "ignore_fields": (
                        "password1",
                        "password2",
                        "new_password1",
                        "new_password2",
                    ),
                },
            },
        )



from datetime import timedelta

from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from product.enum.review import RateEnum, ReviewStatus
from product.models.review import ProductReview


class ProductReviewFilter(filters.FilterSet):
    id = filters.NumberFilter(
        field_name="id",
        lookup_expr="exact",
        help_text=_("Filter by review ID"),
    )
    user = filters.NumberFilter(
        field_name="user_id",
        lookup_expr="exact",
        help_text=_("Filter by user ID"),
    )
    product = filters.NumberFilter(
        field_name="product_id",
        lookup_expr="exact",
        help_text=_("Filter by product ID"),
    )
    status = filters.ChoiceFilter(
        field_name="status",
        choices=ReviewStatus.choices,
        help_text=_("Filter by review status"),
    )
    rate = filters.ChoiceFilter(
        field_name="rate",
        choices=RateEnum.choices,
        help_text=_("Filter by rating"),
    )
    rate_min = filters.NumberFilter(
        field_name="rate",
        lookup_expr="gte",
        help_text=_("Filter by minimum rating"),
    )
    rate_max = filters.NumberFilter(
        field_name="rate",
        lookup_expr="lte",
        help_text=_("Filter by maximum rating"),
    )
    is_published = filters.BooleanFilter(
        field_name="is_published",
        help_text=_("Filter by published status"),
    )
    is_visible = filters.BooleanFilter(
        field_name="is_visible",
        help_text=_("Filter by visibility status"),
    )
    created_after = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="gte",
        help_text=_("Filter reviews created after this date"),
    )
    created_before = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="lte",
        help_text=_("Filter reviews created before this date"),
    )
    published_after = filters.DateTimeFilter(
        field_name="published_at",
        lookup_expr="gte",
        help_text=_("Filter reviews published after this date"),
    )
    published_before = filters.DateTimeFilter(
        field_name="published_at",
        lookup_expr="lte",
        help_text=_("Filter reviews published before this date"),
    )
    comment = filters.CharFilter(
        field_name="translations__comment",
        lookup_expr="icontains",
        help_text=_("Filter by comment content (partial match)"),
    )
    product_name = filters.CharFilter(
        field_name="product__translations__name",
        lookup_expr="icontains",
        help_text=_("Filter by product name (partial match)"),
    )
    user_email = filters.CharFilter(
        field_name="user__email",
        lookup_expr="icontains",
        help_text=_("Filter by user email (partial match)"),
    )
    has_comment = filters.BooleanFilter(
        method="filter_has_comment",
        help_text=_("Filter reviews that have/don't have comments"),
    )
    recent_days = filters.NumberFilter(
        method="filter_recent_days",
        help_text=_("Filter reviews from the last N days"),
    )

    class Meta:
        model = ProductReview
        fields = [
            "id",
            "user",
            "product",
            "status",
            "rate",
            "rate_min",
            "rate_max",
            "is_published",
            "is_visible",
            "created_after",
            "created_before",
            "published_after",
            "published_before",
            "comment",
            "product_name",
            "user_email",
            "has_comment",
            "recent_days",
        ]

    def filter_has_comment(self, queryset, name, value):
        if value is True:
            return queryset.exclude(
                models.Q(translations__comment__isnull=True)
                | models.Q(translations__comment__exact="")
            ).distinct()
        elif value is False:
            return queryset.filter(
                models.Q(translations__comment__isnull=True)
                | models.Q(translations__comment__exact="")
            ).distinct()
        return queryset

    def filter_recent_days(self, queryset, name, value):
        if value and value > 0:
            cutoff_date = timezone.now() - timedelta(days=value)
            return queryset.filter(created_at__gte=cutoff_date)
        return queryset



import contextlib
import json
from io import BytesIO

import pytest
from django.test import RequestFactory
from rest_framework.exceptions import ParseError

from core.api.parsers import (
    NoUnderscoreBeforeNumberCamelCaseFormParser,
    NoUnderscoreBeforeNumberCamelCaseJSONParser,
    NoUnderscoreBeforeNumberCamelCaseMultiPartParser,
)


class TestNoUnderscoreBeforeNumberCamelCaseJSONParser:
    def setup_method(self):
        self.parser = NoUnderscoreBeforeNumberCamelCaseJSONParser()
        self.factory = RequestFactory()

    def test_json_underscoreize_configuration(self):
        assert self.parser.json_underscoreize == {
            "no_underscore_before_number": True
        }

    def test_parse_camel_case_json(self):
        json_data = {
            "firstName": "John",
            "lastName": "Doe",
            "phoneNumber2": "123456789",
            "email": "john@example.com",
        }
        json_string = json.dumps(json_data)
        stream = BytesIO(json_string.encode("utf-8"))

        request = self.factory.post(
            "/", data=json_string, content_type="application/json"
        )
        parser_context = {"request": request}

        result = self.parser.parse(stream, "application/json", parser_context)

        assert "first_name" in result
        assert "last_name" in result
        assert "phone_number2" in result
        assert "email" in result

    def test_parse_invalid_json(self):
        invalid_json = "{'invalid': json}"
        stream = BytesIO(invalid_json.encode("utf-8"))

        request = self.factory.post(
            "/", data=invalid_json, content_type="application/json"
        )
        parser_context = {"request": request}

        with pytest.raises(ParseError):
            self.parser.parse(stream, "application/json", parser_context)

    def test_parse_empty_json(self):
        json_string = "{}"
        stream = BytesIO(json_string.encode("utf-8"))

        request = self.factory.post(
            "/", data=json_string, content_type="application/json"
        )
        parser_context = {"request": request}

        result = self.parser.parse(stream, "application/json", parser_context)
        assert result == {}


class TestNoUnderscoreBeforeNumberCamelCaseFormParser:
    def setup_method(self):
        self.parser = NoUnderscoreBeforeNumberCamelCaseFormParser()
        self.factory = RequestFactory()

    def test_parse_form_data_with_context_override(self):
        form_data = "firstName=John&lastName=Doe&email=john@example.com"
        stream = BytesIO(form_data.encode("utf-8"))

        request = self.factory.post(
            "/",
            data=form_data,
            content_type="application/x-www-form-urlencoded",
        )
        parser_context = {"request": request}

        result = self.parser.parse(
            stream, "application/x-www-form-urlencoded", parser_context
        )

        assert isinstance(result, dict)

    def test_parse_password_fields_ignored(self):
        form_data = "password1=secret&password2=secret&firstName=John"
        stream = BytesIO(form_data.encode("utf-8"))

        request = self.factory.post(
            "/",
            data=form_data,
            content_type="application/x-www-form-urlencoded",
        )
        parser_context = {"request": request}

        result = self.parser.parse(
            stream, "application/x-www-form-urlencoded", parser_context
        )

        assert isinstance(result, dict)

    def test_parse_context_configuration(self):
        stream = BytesIO(b"test=value")
        request = self.factory.post(
            "/",
            data="test=value",
            content_type="application/x-www-form-urlencoded",
        )
        parser_context = {"request": request}

        original_parse = self.parser.__class__.__bases__[0].parse
        context_used = None

        def mock_parse(
            self_inner, stream_inner, media_type_inner, parser_context_inner
        ):
            nonlocal context_used
            context_used = parser_context_inner
            return {}

        self.parser.__class__.__bases__[0].parse = mock_parse

        try:
            self.parser.parse(
                stream, "application/x-www-form-urlencoded", parser_context
            )

            assert "json_underscoreize" in context_used
            assert (
                context_used["json_underscoreize"][
                    "no_underscore_before_number"
                ]
                is True
            )
            assert (
                "password1" in context_used["json_underscoreize"]["ignore_keys"]
            )
            assert (
                "password2" in context_used["json_underscoreize"]["ignore_keys"]
            )
            assert (
                "new_password1"
                in context_used["json_underscoreize"]["ignore_keys"]
            )
            assert (
                "new_password2"
                in context_used["json_underscoreize"]["ignore_keys"]
            )
        finally:
            self.parser.__class__.__bases__[0].parse = original_parse


class TestNoUnderscoreBeforeNumberCamelCaseMultiPartParser:
    def setup_method(self):
        self.parser = NoUnderscoreBeforeNumberCamelCaseMultiPartParser()
        self.factory = RequestFactory()

    def test_parse_multipart_data(self):
        stream = BytesIO(
            b'--boundary\r\nContent-Disposition: form-data; name="test"\r\n\r\nvalue\r\n--boundary--'
        )

        request = self.factory.post(
            "/", data={}, content_type="multipart/form-data; boundary=boundary"
        )
        parser_context = {"request": request}

        with contextlib.suppress(Exception):
            self.parser.parse(stream, "multipart/form-data", parser_context)

    def test_parse_context_configuration_multipart(self):
        stream = BytesIO(b"test data")
        request = self.factory.post(
            "/", data={}, content_type="multipart/form-data"
        )
        parser_context = {"request": request}

        original_parse = self.parser.__class__.__bases__[0].parse
        context_used = None

        def mock_parse(
            self_inner, stream_inner, media_type_inner, parser_context_inner
        ):
            nonlocal context_used
            context_used = parser_context_inner
            return {}

        self.parser.__class__.__bases__[0].parse = mock_parse

        try:
            self.parser.parse(stream, "multipart/form-data", parser_context)

            assert "json_underscoreize" in context_used
            assert (
                context_used["json_underscoreize"][
                    "no_underscore_before_number"
                ]
                is True
            )
            assert (
                "password1" in context_used["json_underscoreize"]["ignore_keys"]
            )
            assert (
                "password2" in context_used["json_underscoreize"]["ignore_keys"]
            )
            assert (
                "new_password1"
                in context_used["json_underscoreize"]["ignore_keys"]
            )
            assert (
                "new_password2"
                in context_used["json_underscoreize"]["ignore_keys"]
            )

            assert (
                "password1"
                in context_used["json_underscoreize"]["ignore_fields"]
            )
            assert (
                "password2"
                in context_used["json_underscoreize"]["ignore_fields"]
            )
            assert (
                "new_password1"
                in context_used["json_underscoreize"]["ignore_fields"]
            )
            assert (
                "new_password2"
                in context_used["json_underscoreize"]["ignore_fields"]
            )
        finally:
            self.parser.__class__.__bases__[0].parse = original_parse

    def test_parser_inheritance(self):
        assert hasattr(self.parser, "parse")
        assert callable(self.parser.parse)
